@@ -22,6 +22,7 @@ void marhythm_show_about_dialog()
 	GtkWidget *aboutDialogContentArea;
 	GtkWidget *applicationLabel;
 	GtkWidget *authorLabel;
+	GtkWidget *thanksLabel;
 	GtkWidget *webLabel;
 	
 	PangoAttrList *applicationLabelAttributeList;
@@ -71,35 +72,42 @@ void marhythm_show_about_dialog()
 						FALSE,
 						FALSE,
 						0);
+						
+	thanksLabel = gtk_label_new( MARHYTHM_SPECIAL_THANKS_TEXT );
+	
+	gtk_box_pack_start( GTK_BOX( aboutDialogContentArea ),
+						thanksLabel,
+						FALSE,
+						FALSE,
+						0);
+						
 	
 	gtk_widget_show_all( aboutDialog );
 	
 	
-	
 	gtk_box_set_spacing( GTK_BOX( aboutDialogContentArea ),
 								10);
 	gtk_dialog_run( GTK_DIALOG( aboutDialog ) );
 	
 	
-	
-	
 	gtk_widget_destroy( aboutDialog );
 	
-	
 	pango_attribute_destroy( applicationLabelSizeAttribute );
-	
 }
 
-void marhythm_update_biorhythm_widgets( MarhythmBiorhythmUpdatableWidgets *biorhythmUpdatableWidgets, MarhythmBiorhythmValues biorhythmValues, GDate *selectedDate )
+void marhythm_update_biorhythm_widgets( MarhythmBiorhythmUpdatableWidgets *biorhythmUpdatableWidgets, MarhythmBiorhythmIcons *biorhythmIcons, MarhythmBiorhythmValues biorhythmValues, GDate *selectedDate )
 {
 	gchar selectedDateString[128];
 	
 	marhythm_update_stars_from_biorhythm_value( biorhythmUpdatableWidgets->physicalStarWidgets,
-		biorhythmValues.physical );
+		biorhythmValues.physical,
+		biorhythmIcons );
 	marhythm_update_stars_from_biorhythm_value( biorhythmUpdatableWidgets->emotionalStarWidgets,
-		biorhythmValues.emotional );
+		biorhythmValues.emotional,
+		biorhythmIcons );
 	marhythm_update_stars_from_biorhythm_value( biorhythmUpdatableWidgets->intellectualStarWidgets,
-		biorhythmValues.intellectual );
+		biorhythmValues.intellectual,
+		biorhythmIcons );
 	
 	g_date_strftime( (gchar*) selectedDateString, sizeof( selectedDateString ), "%x", selectedDate );
 	
@@ -110,6 +118,7 @@ void marhythm_update_biorhythm_widgets( MarhythmBiorhythmUpdatableWidgets *biorh
 		"highlight-date",
 		selectedDate,
 		NULL );
+		
 }
 
 void marhythm_update_biorhythm_screen( MarhythmData *applicationData )
@@ -127,27 +136,28 @@ void marhythm_update_biorhythm_screen( MarhythmData *applicationData )
 	biorhythmValues.intellectual = mr_calculate_cycle_energy_from_date( applicationData->currentContactBirthDate,
 								applicationData->currentSelectedDate,
 								MR_INTELLECTUAL_CYCLE );
-	
+	/*
 	g_object_set( G_OBJECT( applicationData->biorhythmUpdatableWidgets.biorhythmChart ),
 		"reference-date",
 		applicationData->currentReferenceDate,
 		NULL );
-	
+	*/
 	marhythm_update_biorhythm_widgets( &applicationData->biorhythmUpdatableWidgets,
+									&applicationData->biorhythmIcons,
 									biorhythmValues,
 									applicationData->currentSelectedDate );
 }
 
-void marhythm_update_stars_from_biorhythm_value( GtkWidget* widgetsArray[], gfloat value )
+void marhythm_update_stars_from_biorhythm_value( GtkWidget* widgetsArray[], gfloat value, MarhythmBiorhythmIcons *biorhythmIcons )
 {
 	gint enabledStars = 0;
 	
 	if( value < 15.0 && value > -15.0 )
 	{
-		gtk_image_set_from_file( GTK_IMAGE( widgetsArray[0] ), MARHYTHM_IMAGE_STAR_CRITICAL_PATH );
-		gtk_image_set_from_file( GTK_IMAGE( widgetsArray[1] ), MARHYTHM_IMAGE_STAR_CRITICAL_PATH );
-		gtk_image_set_from_file( GTK_IMAGE( widgetsArray[2] ), MARHYTHM_IMAGE_STAR_CRITICAL_PATH );
-		gtk_image_set_from_file( GTK_IMAGE( widgetsArray[3] ), MARHYTHM_IMAGE_STAR_CRITICAL_PATH );
+		gtk_image_set_from_pixbuf( GTK_IMAGE( widgetsArray[0] ), biorhythmIcons->critical );
+		gtk_image_set_from_pixbuf( GTK_IMAGE( widgetsArray[1] ), biorhythmIcons->critical );
+		gtk_image_set_from_pixbuf( GTK_IMAGE( widgetsArray[2] ), biorhythmIcons->critical );
+		gtk_image_set_from_pixbuf( GTK_IMAGE( widgetsArray[3] ), biorhythmIcons->critical );
 	}
 	else
 	{
@@ -185,28 +195,28 @@ void marhythm_update_stars_from_biorhythm_value( GtkWidget* widgetsArray[], gflo
 		
 		for( i = 0; i < enabledStars; i++  )
 		{
-			gtk_image_set_from_file( GTK_IMAGE( widgetsArray[i] ), MARHYTHM_IMAGE_STAR_ENABLED_PATH );
+			gtk_image_set_from_pixbuf( GTK_IMAGE( widgetsArray[i] ), biorhythmIcons->enabled );
 		}
 		
 		
 		for( i = enabledStars; i < 4; i++ )
 		{
-			gtk_image_set_from_file( GTK_IMAGE( widgetsArray[i] ), MARHYTHM_IMAGE_STAR_DISABLED_PATH );
+			gtk_image_set_from_pixbuf( GTK_IMAGE( widgetsArray[i] ), biorhythmIcons->disabled );
 		}
 		
 	}
 }
 
-void marhythm_create_stars_from_biorhythm_value( GtkWidget* widgetsArray[], gfloat value )
+void marhythm_create_stars_from_biorhythm_value( GtkWidget* widgetsArray[], gfloat value, MarhythmBiorhythmIcons *biorhythmIcons )
 {
 	gint enabledStars = 0;
 	
 	if( value < 15.0 && value > -15.0 )
 	{
-		widgetsArray[0] = gtk_image_new_from_file( MARHYTHM_IMAGE_STAR_CRITICAL_PATH );
-		widgetsArray[1] = gtk_image_new_from_file( MARHYTHM_IMAGE_STAR_CRITICAL_PATH );
-		widgetsArray[2] = gtk_image_new_from_file( MARHYTHM_IMAGE_STAR_CRITICAL_PATH );
-		widgetsArray[3] = gtk_image_new_from_file( MARHYTHM_IMAGE_STAR_CRITICAL_PATH );
+		widgetsArray[0] = gtk_image_new_from_pixbuf( biorhythmIcons->critical );
+		widgetsArray[1] = gtk_image_new_from_pixbuf( biorhythmIcons->critical );
+		widgetsArray[2] = gtk_image_new_from_pixbuf( biorhythmIcons->critical );
+		widgetsArray[3] = gtk_image_new_from_pixbuf( biorhythmIcons->critical );
 	}
 	else
 	{
@@ -244,13 +254,13 @@ void marhythm_create_stars_from_biorhythm_value( GtkWidget* widgetsArray[], gflo
 		
 		for( i = 0; i < enabledStars; i++  )
 		{
-			widgetsArray[i] = gtk_image_new_from_file( MARHYTHM_IMAGE_STAR_ENABLED_PATH );
+			widgetsArray[i] = gtk_image_new_from_pixbuf( biorhythmIcons->enabled );
 		}
 		
 		
 		for( i = enabledStars; i < 4; i++ )
 		{
-			widgetsArray[i] = gtk_image_new_from_file( MARHYTHM_IMAGE_STAR_DISABLED_PATH );
+			widgetsArray[i] = gtk_image_new_from_pixbuf( biorhythmIcons->disabled );
 		}
 		
 	}
@@ -277,7 +287,13 @@ marhythm_display_biorhythm( MarhythmData *applicationData, OssoABookContact *con
     contactBirthDate = ( EContactDate* )  e_contact_get ( E_CONTACT ( contact ),
                                                          E_CONTACT_BIRTH_DATE );
 
-    birthDate = g_date_new_dmy( contactBirthDate->day, contactBirthDate->month , contactBirthDate->year );
+    birthDate = g_date_new_dmy( contactBirthDate->day, contactBirthDate->month , contactBirthDate->year );    
+    
+    
+    /* Don't need the EContactDate anymore */
+    e_contact_date_free( contactBirthDate );
+
+    
     
     applicationData->currentContactBirthDate = birthDate;
     
@@ -353,10 +369,11 @@ marhythm_display_biorhythm( MarhythmData *applicationData, OssoABookContact *con
     
     
     MarhythmBiorhythmUpdatableWidgets *biorhythmUpdatableWidgets = &applicationData->biorhythmUpdatableWidgets;
+    MarhythmBiorhythmIcons *biorhythmIcons = &applicationData->biorhythmIcons;
     
-    marhythm_create_stars_from_biorhythm_value( biorhythmUpdatableWidgets->physicalStarWidgets, physicalValue );
-    marhythm_create_stars_from_biorhythm_value( biorhythmUpdatableWidgets->emotionalStarWidgets, emotionalValue );
-    marhythm_create_stars_from_biorhythm_value( biorhythmUpdatableWidgets->intellectualStarWidgets, intellectualValue );
+    marhythm_create_stars_from_biorhythm_value( biorhythmUpdatableWidgets->physicalStarWidgets, physicalValue, biorhythmIcons );
+    marhythm_create_stars_from_biorhythm_value( biorhythmUpdatableWidgets->emotionalStarWidgets, emotionalValue, biorhythmIcons );
+    marhythm_create_stars_from_biorhythm_value( biorhythmUpdatableWidgets->intellectualStarWidgets, intellectualValue, biorhythmIcons );
 
     physicalLabel = gtk_label_new( "Physical" );
     emotionalLabel = gtk_label_new( "Emotional" );
@@ -815,6 +832,11 @@ marhythm_display_biorhythm( MarhythmData *applicationData, OssoABookContact *con
                             NULL);
 
     gtk_widget_show_all( personalBiorhythmWindow );
+    
+    g_signal_connect_after( G_OBJECT( personalBiorhythmWindow ),
+							"destroy",
+							G_CALLBACK( biorhythmWindowDestroyed ),
+							applicationData);    
 
 }
 
@@ -824,10 +846,25 @@ void
 marhythm_display_personal_biorhythm( MarhythmData *applicationData )
 {
     OssoABookSelfContact *selfContact;
+    EContactDate *contactBirthDate;
 
-    selfContact = osso_abook_self_contact_get_default();
+    selfContact = osso_abook_self_contact_get_default(); 
+    
+    
+    contactBirthDate = ( EContactDate* )  e_contact_get ( E_CONTACT ( selfContact ),
+                                                         E_CONTACT_BIRTH_DATE );
+	e_contact_date_free( contactBirthDate );
+	
+	if( contactBirthDate != NULL )
+    {
+        marhythm_display_biorhythm( applicationData, OSSO_ABOOK_CONTACT( selfContact ) );
+    }
+    else
+    {
+        hildon_banner_show_information ( applicationData->mainWindow, NULL, "Your birthdate has not been specified");
+    }
 
-   marhythm_display_biorhythm( applicationData, OSSO_ABOOK_CONTACT( selfContact ) );
+	
 }
 
 void
@@ -935,6 +972,10 @@ void marhythm_set_default_current_reference_date( MarhythmData *applicationData
     GTimeVal currentTime;    
     g_get_current_time ( &currentTime );
 
+	if( applicationData->currentReferenceDate != NULL )
+	{
+		g_date_free( applicationData->currentReferenceDate );
+	}
 
     currentDate = g_date_new ();
 
@@ -1000,6 +1041,36 @@ void createApplicationMenu( MarhythmData *applicationData )
     gtk_widget_show_all (GTK_WIDGET (applicationMenu) );    
 }
 
+void marhythm_load_biorhythm_icons( MarhythmData *applicationData )
+{
+	applicationData->biorhythmIcons.enabled = gdk_pixbuf_new_from_file(MARHYTHM_IMAGE_STAR_ENABLED_PATH, NULL);
+	applicationData->biorhythmIcons.disabled = gdk_pixbuf_new_from_file(MARHYTHM_IMAGE_STAR_DISABLED_PATH, NULL);
+	applicationData->biorhythmIcons.critical = gdk_pixbuf_new_from_file(MARHYTHM_IMAGE_STAR_CRITICAL_PATH, NULL);
+}
+
+
+void marhythm_free_data( MarhythmData *applicationData )
+{
+	if( applicationData->currentReferenceDate != NULL )
+	{
+		g_date_free( applicationData->currentReferenceDate );
+	}
+	
+	if( applicationData->currentSelectedDate != NULL )
+	{
+		g_date_free( applicationData->currentSelectedDate );
+	}
+	
+	if( applicationData->currentContactBirthDate != NULL )
+	{
+		g_date_free( applicationData->currentContactBirthDate );
+	}
+	
+	gdk_pixbuf_unref( applicationData->biorhythmIcons.critical );
+	gdk_pixbuf_unref( applicationData->biorhythmIcons.disabled );
+	gdk_pixbuf_unref( applicationData->biorhythmIcons.enabled );
+}
+
 void marhythm_init( MarhythmData *applicationData  )
 {
     HildonProgram *program;	
@@ -1009,6 +1080,11 @@ void marhythm_init( MarhythmData *applicationData  )
 
 	mainWindow = hildon_stackable_window_new();
     hildon_program_add_window (program, HILDON_WINDOW( mainWindow ) );
+    
+    // Setting default values
+    applicationData->currentContactBirthDate = NULL;
+    applicationData->currentReferenceDate = NULL;
+    applicationData->currentSelectedDate = NULL;
 
     // Linking data
     applicationData->mainWindow = mainWindow;
@@ -1022,8 +1098,12 @@ void marhythm_init( MarhythmData *applicationData  )
     //Connecting signals
     connectSignals( applicationData );
 
-    // Setting default reference date
+    //Setting default reference date
     marhythm_set_default_current_reference_date( applicationData );
+    
+    //Loading biorhythm images in memory
+    marhythm_load_biorhythm_icons( applicationData );
+
 
 	gtk_widget_show_all( mainWindow );
 
