@@ -46,6 +46,9 @@ static guint mr_biorhythm_chart_signals[LAST_SIGNAL] = { 0 };
 
 static gboolean mr_biorhythm_chart_expose (GtkWidget *biorhythm, GdkEventExpose *event);
 
+static void mr_biorhythm_chart_finalize(GObject *object);
+
+
 static void drawCacheChart( GtkWidget *biorhythm );
 
 
@@ -228,6 +231,9 @@ mr_biorhythm_chart_class_init (MrBiorhythmChartClass *class)
              0);
 
 	widget_class->expose_event = mr_biorhythm_chart_expose;
+	obj_class->finalize = mr_biorhythm_chart_finalize;
+
+	
     g_type_class_add_private (obj_class, sizeof (MrBiorhythmChartPrivate));
 }
 
@@ -375,6 +381,10 @@ draw (GtkWidget *biorhythm, cairo_t *cr)
                 finishDay = g_date_days_between( tempDate, endDate  );
             }
 
+			/* Freeing don't needed dates */
+			g_date_free( limitTempDate ); 
+			
+
             visibleDays = finishDay - startDay;
 
             totalTextWidth = (monthTextWidth / PANGO_SCALE) * 3;
@@ -402,6 +412,8 @@ draw (GtkWidget *biorhythm, cairo_t *cr)
 
     }
 
+	/* We don't need endDate anymore */
+	g_date_free( endDate );
     
     pango_font_description_free( fontDescription );
     g_string_free( dayNumberString, TRUE );  
@@ -562,3 +574,16 @@ mr_biorhythm_chart_new (void)
 {
 	return g_object_new (MR_TYPE_BIORHYTHM_CHART, NULL);
 }
+
+static void mr_biorhythm_chart_finalize( GObject *object)
+{
+	MrBiorhythmChart *biorhythmChart = MR_BIORHYTHM_CHART(object);
+	
+	MrBiorhythmChartPrivate *private;
+    private = MR_BIORHYTHM_CHART_GET_PRIVATE (biorhythmChart);
+    
+    g_date_free( private->birth_date );
+    g_date_free( private->highlight_date );
+    g_date_free( private->reference_date );
+    cairo_surface_destroy( private->cachedChart );
+}
